type uchar = int

module Utf8 :
sig
  include Custom.T
  type uindex

  val mk_check: string -> t option
  val to_string: t -> string
  val print: Format.formatter -> t -> unit

  val end_index: t -> uindex
  val start_index: t -> uindex
  val equal_index: uindex -> uindex -> bool
  val mk: string -> t
  val mk_latin1: string -> t
  val mk_char: int -> t
  val mk_idx: int -> uindex
  val get_str: t -> string
  val get_idx: uindex -> int

  val get: t -> uindex -> uchar
  val advance: t -> uindex -> uindex
  val next: t -> uindex -> uchar * uindex 

  val concat: t -> t -> t

  val store: Buffer.t -> uchar -> unit
  val copy: Buffer.t -> t -> uindex -> uindex -> unit
  val get_substr: t -> uindex -> uindex -> string

  val serialize_sub: Serialize.Put.t -> t -> uindex -> uindex -> unit

  module Pcre:
    (* Wrappers for some Pcre functions working on Utf8.t values.
     * Given Pcre.regexp regular expressions should be compiled with `UTF8 flag
     * or with pcre_regexp below *)
  sig
    val pcre_regexp: string -> Pcre.regexp  (* compile using `UTF8 flag *)

    val pcre_replace: rex:Pcre.regexp -> ?templ:t -> t -> t
    val pcre_extract: rex:Pcre.regexp -> t -> t array
    val pcre_split: rex:Pcre.regexp -> t -> t list
  end

end
