(**************************************************************************)
(*  The CDuce compiler                                                    *)
(*  Alain Frisch <Alain.Frisch@ens.fr> and the CDuce team                 *)
(*  Copyright 2003,2004 (see LICENSE for details)                         *)
(**************************************************************************)

open Location
open Ast
open Ident
open Printf

open Parser.Hook
open Query
#load "pa_extend.cmo";;


let exp pos e = LocatedExpr (loc_of_pos pos,e)

let cst_nil =  Const Sequence.nil_cst
let parse_ident = U.mk

let id_dummy = ident (U.mk "$$$")

let label = parse_ident

let rec multi_prod loc = function
  | [ x ] -> x
  | x :: l -> mk loc (Prod (x, multi_prod loc l))
  | [] -> assert false
 
EXTEND 
  GLOBAL: expr pat keyword;


  expr: [
    "top" [
     "select"; e = expr;
     "from";l = LIST1 [ x= pat ; "in"; e = expr -> (x,e)] SEP "," ;
      z=OPT["where"  ; w = cond -> w]  -> 
	let (condi,fin) =
	  match z with
              Some w -> 
                (w, exp loc 
		   (Parser.if_then_else (Query.ast_of_bool(w,loc)) 
		      (Pair (e,cst_nil)) 
		      cst_nil))
            | None -> (True, exp loc (Pair(e,cst_nil)))
	in 
	if !Query.nooptim 
        then Query.select(loc,fin,l)
      	else Query.selectOpt(loc,Pair (e,cst_nil),l,condi)
   | 
      e = expr; "/@"; 
      a = [IDENT|keyword]-> (* projection sur 1 attribut *)
        let tag = mk loc (Internal(Types.atom Atoms.any)) in
        let any = mk loc (Internal(Types.any)) in
        let att = mk loc (Record(true,[(label a,mk loc (PatVar(U.mk "$$$")))]))in
        (*let ct= mk loc (Regexp(Elem any , any)) in *)
        let p = mk loc(XmlT (tag,multi_prod loc[att;any])) in
        let t =(p, Pair(Var (Id.value id_dummy),cst_nil)) 
        in exp loc (Transform (e,[t]))
     | e = expr; "//" ; p = pat -> (* projections sur tous les descendants *)
       let assign= 
       exp loc ( Apply (Dot (Var(U.mk"$stack"), U.mk"set"),
           (Op("@",
	  [(Apply(Dot(Var(U.mk"$stack"),U.mk"get"),cst_nil));Pair(Var(U.mk"$$$"),cst_nil)]))))
       in let branche=Pair(Var (Id.value id_dummy),cst_nil)
       in let branches= exp loc (Match(assign,[pat_nil,branche]))
       in let xt=exp loc (Xtrans (e,[(mk loc(And(mk loc(PatVar(U.mk "$$$")),p))),branches]))
       in let rf=exp loc(Ref(cst_nil,mk loc (Regexp 
	  (Star(Elem(p)),pat_nil))))
       in exp loc(Match(rf,[mk loc(PatVar(U.mk"$stack")),
               exp loc(Match(xt,
	          [mk loc(Internal Types.any),
	          exp loc (Apply(Dot(Var(U.mk"$stack"),U.mk"get"),cst_nil))]))
	       ]))   	      
     
     (* equivalent as:
     
      let $stack=ref [p*] []
      in let _ = xtransform e with
                  $$$ & p -> $stack := [ !$stack @ $$$] ; [$$$]
	in !stack;; 
	
     NB: order is inversed	
	*)	  
      ]
    ];
   
  cond:
      [ [ a = expr -> 
	  (match a with
             | LocatedExpr(_, Atom at) ->
	         (match U.get_str at with
                  | "true" -> Query.True
                  | "false" -> Query.False
                  | _ -> Query.Varb a)
             | _ -> Query.Varb a)
       |"not"; a = cond -> Query.Not(a)
       | a = cond ; "or" ; b = cond -> Query.Ou(a,b)
       | a = cond ; "and" ; b = cond -> Query.Et(a,b)
       | "(" ; a=cond ; ")" -> a
      ]
    ];


  keyword: [ [ a = [ "select" | "from" | "where" ] -> a ] ];
END




