(**************************************************************************)
(*  The CDuce compiler                                                    *)
(*  Alain Frisch <Alain.Frisch@ens.fr> and the CDuce team                 *)
(*  Copyright 2003,2004 (see LICENSE for details)                         *)
(**************************************************************************)

open Pxp_yacc
open Pxp_lexer_types
open Pxp_types

let pxp_handle_event = function
  | E_start_tag (name,att,_) -> Load_xml.start_element_handler name att
  | E_char_data data -> Load_xml.text_handler data 
  | E_end_tag (_,_) -> Load_xml.end_element_handler ()
  | _ -> ()

let pxp_config = 
  { default_config with 
      (* warner = new warner; *)
      encoding = `Enc_utf8;
      store_element_positions = false;
      drop_ignorable_whitespace = true
  }


let load_pxp s =
  try
    let src = 
      match Url.process s with 
        | Url.Url s -> from_string s
        | Url.Filename s -> from_file s in
    let mgr = create_entity_manager pxp_config src in
    process_entity pxp_config (`Entry_document[`Extend_dtd_fully]) mgr pxp_handle_event;
  with exn ->
    raise (Location.Generic (Pxp_types.string_of_exn exn))

let () = 
  Config.register 
    "pxp" 
    "PXP XML parser"
    (fun () -> Load_xml.xml_parser := load_pxp)
